#include <avr/io.h>
#include <avr/interrupt.h>
#define F_CPU 16000000
#include "line_follower.h"
#include <util/delay.h>

static inline void moveForward(float speed, enum motor_t type)
{
	switch (type) {
	case MOTOR_1:
		OCR1A = (int)(REF_SPEED * speed);
		break;
	case MOTOR_2:
		OCR1B = (int)(REF_SPEED * speed);
		break;
	}
}

static inline void moveBackward(float speed, enum motor_t type)
{
	switch (type) {
	case MOTOR_1:
		PORTD |= _BV(IN1);
		OCR1A = (int)(REF_SPEED * speed);
		break;
	case MOTOR_2:
		PORTD |= _BV(IN2);
		OCR1B = (int)(REF_SPEED * speed);
		break;
	}
}

static inline void stop(enum motor_t type)
{
	switch (type) {
	case MOTOR_1:
		OCR1A = 0;
		break;
	case MOTOR_2:
		OCR1B = 0;
		break;
	}
}

static void turnRight(float speed)
{
	stop(MOTOR_2);
	moveForward(speed, MOTOR_1);
}

static void turnLeft(float speed)
{
	stop(MOTOR_1);
	moveForward(speed, MOTOR_2);
}

static void init(void)
{

	/*set 16-bit timer */
	TCCR1A = _BV(COM1A1) | _BV(COM1B1);
	TCCR1B = _BV(WGM13) | _BV(CS11);
	ICR1 = 1000;

	DDRA = DDRC = 0x0;
	PINA = PINC = PORTD = PORTA = PORTC = 0x00;

    DDRD = 0xFF;
	PORTD |= _BV(PWM1) | _BV(EN1) | _BV(PWM2) | _BV(EN2);
}


int main()
{
	unsigned int leftSen, rightSen;
	enum dir_t lastOnRoad;
	int curbureCoefficient = 0;
	init();

	lastOnRoad = LEFT;

	while(LF_TRUE) {
		leftSen = PINA & _BV(PA6);
		rightSen = PINC & _BV(PC6);

		if (!leftSen && !rightSen) {
			moveForward(1 - (curbureCoefficient / (MAX_COEF * 2)), MOTOR_1);
			moveForward(1 - (curbureCoefficient / (MAX_COEF * 2)), MOTOR_2);
			DEC(curbureCoefficient);
		} else if (!leftSen && rightSen) {
			turnRight(2 - (curbureCoefficient / (MAX_COEF * 2)));
			INC(curbureCoefficient);
		} else if (leftSen && !rightSen) {
			turnLeft(2 - (curbureCoefficient / (MAX_COEF * 2)));
			INC(curbureCoefficient);
		} else {
			moveForward(0.5, MOTOR_1);
			moveForward(0.5, MOTOR_2);
			INC(curbureCoefficient);
		}
	}

    return 0;
}
